@@ -17,13 +17,11 @@ from chaste_codegen._script_utils import write_file
 
 # pass --<command_line_tag> to select this model type
 
-# Pycml generated BackwardEuler with lookup tables by default,
-# so we introduced BackwardEulerNoLut for the version without lookup table
 TRANSLATORS = OrderedDict(
     [('normal', (cg.NormalChasteModel, 'FromCellML', '', True)),
      ('cvode', (cg.CvodeChasteModel, 'FromCellMLCvode', 'Cvode', True)),
      ('cvode-data-clamp', (cg.CvodeChasteModel, 'FromCellMLCvodeDataClamp', 'CvodeDataClamp', True)),
-     ('backward-euler', (cg.BackwardEulerModel, 'FromCellMLBackwardEulerNoLut', 'BackwardEulerNoLut', False)),
+     ('backward-euler', (cg.BackwardEulerModel, 'FromCellMLBackwardEuler', 'BackwardEuler', False)),
      ('rush-larsen', (cg.RushLarsenModel, 'FromCellMLRushLarsen', 'RushLarsen', False)),
      ('grl1', (cg.GeneralisedRushLarsenFirstOrderModel, 'FromCellMLGRL1', 'GRL1', False)),
      ('grl2', (cg.GeneralisedRushLarsenSecondOrderModel, 'FromCellMLGRL2', 'GRL2', False))])
@@ -32,7 +30,7 @@ TRANSLATORS_OPT = OrderedDict(
     [('normal', (cg.OptChasteModel, 'FromCellMLOpt', 'Opt', True)),
      ('cvode', (cg.OptCvodeChasteModel, 'FromCellMLCvodeOpt', 'CvodeOpt', True)),
      ('cvode-data-clamp', (cg.OptCvodeChasteModel, 'FromCellMLCvodeDataClampOpt', 'CvodeDataClampOpt', True)),
-     ('backward-euler', (cg.BackwardEulerOptModel, 'FromCellMLBackwardEuler', 'BackwardEuler', False)),
+     ('backward-euler', (cg.BackwardEulerOptModel, 'FromCellMLBackwardEulerOpt', 'BackwardEulerOpt', False)),
      ('rush-larsen', (cg.RushLarsenOptModel, 'FromCellMLRushLarsenOpt', 'RushLarsenOpt', False)),
      ('grl1', (cg.GeneralisedRushLarsenFirstOrderModelOpt, 'FromCellMLGRL1Opt', 'GRL1', False)),
      ('grl2', (cg.GeneralisedRushLarsenSecondOrderModelOpt, 'FromCellMLGRL2Opt', 'GRL2', False))])
@@ -111,6 +109,9 @@ def process_command_line():
                             '(optional). --lookup-table can be added multiple times to indicate multiple lookup tables'
                             '. Please note: Can only be used in combination with --opt. If the arguments are omitted, '
                             'following defaults will be used: %s.' % print_default_lookup_params())
+    group.add_argument('--use-model-factory', action='store_true', default=False,
+                       help='Make use of ModelFactoy method to allow creating models by name. '
+                       'Requires ModelFactory.hpp/cpp found in the ApPredict project.')
 
     # process options
     args = parser.parse_args()
@@ -185,6 +186,7 @@ def process_command_line():
             args.class_name = ('Dynamic' if args.dynamically_loadable else 'Cell') + model_name_from_file +\
                 translator[1]
 
+        args.cellml_base = outfile_base
         if not args.outfile:
             outfile_base += translator[2]
 
