@@ -19,7 +19,6 @@ from chaste_codegen import LOGGER, CodegenError
 from chaste_codegen._math_functions import MATH_FUNC_SYMPY_MAPPING
 from chaste_codegen._optimize import optimize_expr_for_c_output
 from chaste_codegen._rdf import OXMETA, PYCMLMETA, get_variables_transitively
-from chaste_codegen._singularity_fixes import fix_singularity_equations
 
 
 MEMBRANE_VOLTAGE_INDEX = 0  # default index for voltage in state vector
@@ -32,18 +31,25 @@ STIM_PARAM_TAGS = (('membrane_stimulus_current_amplitude', 'uA_per_cm2', True),
                    ('membrane_stimulus_current_end', 'millisecond', False))
 
 
-def load_model_with_conversions(model_file, use_modifiers=False, quiet=False, fix_singularities=True):
+def load_model_with_conversions(model_file, use_modifiers=False, quiet=False, skip_singularity_fixes=False):
     if quiet:
         LOGGER.setLevel(logging.ERROR)
     try:
         model = cellmlmanip.load_model(model_file)
     except Exception as e:
         raise CodegenError('Could not load cellml model: \n    ' + str(e))
-    add_conversions(model, use_modifiers=use_modifiers, fix_singularities=fix_singularities)
+    if not skip_singularity_fixes:
+        V = model.get_variable_by_ontology_term((OXMETA, 'membrane_voltage'))
+        tagged = set(model.get_variables_by_rdf((PYCMLMETA, 'modifiable-parameter'), 'yes', sort=False))
+        annotated = set(filter(lambda q: model.has_ontology_annotation(q, OXMETA), model.variables()))
+        excluded = (tagged | annotated) - set(model.get_derived_quantities(sort=False))
+
+        model.remove_fixable_singularities(V, exclude=excluded)
+    add_conversions(model, use_modifiers=use_modifiers)
     return model
 
 
-def add_conversions(model, use_modifiers=True, fix_singularities=True):
+def add_conversions(model, use_modifiers=True):
     # We are adding attributes to the model from cellmlmanip. This could break if the api changes
     # The check  below guards against this
     attrs_added = ('conversion_units', 'stimulus_units', 'time_variable', 'state_vars', 'membrane_voltage_var',
@@ -87,10 +93,6 @@ def add_conversions(model, use_modifiers=True, fix_singularities=True):
     # Retrieve stimulus current parameters so we can exclude these from modifiers etc.
     model.modifiable_parameters = _get_modifiable_parameters(model)
 
-    # Fix singularities if desired
-    if fix_singularities:
-        fix_singularity_equations(model, model.membrane_voltage_var, model.modifiable_parameters)
-
     # Get Capactiatnce & stimulus parameters
     model.membrane_capacitance = _get_membrane_capacitance(model)
     model.stimulus_params, model.stimulus_equations = _get_stimulus(model)
@@ -179,6 +181,7 @@ def _add_units(model):
     uA_per_cm2 = units.add_unit('uA_per_cm2', 'ampere / 1e6 / (meter * 1e-2)**2')
     uA_per_uF = units.add_unit('uA_per_uF', 'ampere / 1e6 / (farad * 1e-6)')
     uA = units.add_unit('uA', 'ampere / 1e6')
+    units.add_unit('uF_per_cm2', 'ampere / 1e6 / (meter * 1e-2)**2')
     units.add_unit('uF', 'farad / 1e6')
     units.add_unit('millisecond', 'second / 1e3')
     units.add_unit('millimolar', 'mole / 1e3 / litre')
@@ -280,16 +283,46 @@ def _get_modifiable_parameters(model):
 
 
 def _get_membrane_capacitance(model):
-    """ Find membrane_capacitance if the model has it and convert it to uF if necessary"""
+    """ Find membrane_capacitance if the model has it and convert it to uF / uF_per_cm2 if necessary
+        Try to convert the capacitance and converts it to appropriate units.
+        see: https://chaste.cs.ox.ac.uk/trac/ticket/1364
+
+        units converted to:
+        Dimensions of current        Dimensions of capacitance
+        amps per unit area           farads per unit area
+        amps per unit capacitance    We don't care
+        amps                         farads
+    """
     try:
         capacitance = model.get_variable_by_ontology_term((OXMETA, 'membrane_capacitance'))
-        return model.convert_variable(capacitance, model.conversion_units.get_unit('uF'), DataDirectionFlow.OUTPUT)
     except KeyError:
         LOGGER.info('The model has no capacitance tagged.')
         return None
+
+    try:
+        capacitance = model.convert_variable(capacitance, model.conversion_units.get_unit('uF'),
+                                             DataDirectionFlow.OUTPUT)
     except DimensionalityError:
-        LOGGER.warning('The model has capacitance in incompatible units, skipping.')
-        return None
+        try:
+            capacitance = model.convert_variable(capacitance, model.conversion_units.get_unit('uF_per_cm2'),
+                                                 DataDirectionFlow.OUTPUT)
+        except DimensionalityError:
+            pass
+
+    # Check units match up with what is expected
+    if model.membrane_stimulus_current_orig is not None:
+        uA_dim = model.conversion_units.get_unit('uA').dimensionality
+        uA_per_cm2_dim = model.conversion_units.get_unit('uA_per_cm2').dimensionality
+        uF_dim = model.conversion_units.get_unit('uF').dimensionality
+        uF_per_cm2_dim = model.conversion_units.get_unit('uF_per_cm2').dimensionality
+
+        current_dim = model.membrane_stimulus_current_orig.units.dimensionality
+        capac_dim = capacitance.units.dimensionality
+
+        if (current_dim == uA_dim and not capac_dim == uF_dim) or \
+                (current_dim == uA_per_cm2_dim and not capac_dim == uF_per_cm2_dim):
+            LOGGER.warning(model.name + ' The model has capacitance in incompatible units.')
+    return capacitance
 
 
 def _get_stimulus(model):
